/*------------------------------------------------------------------------------
--  This file is a part of the libuc, microcontroler library
--  Copyright (C) 2013, Alexis Jeandet
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                 Author : Alexis Jeandet
--                   Mail : alexis.jeandet@member.fsf.org
-------------------------------------------------------------------------------*/
#ifndef THREAD_H
#define THREAD_H
#include <stdint.h>
#include <uhandle.h>
#include <streamdevices.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 This structure holds the thread context. To do so, you need to keep a track of the programm counter,
 to change the stack pointer for each thread and to save all registers needing to be saved. Since the
 number of regiter to be saved vary from one processor to an other, it's easier to save them in the
 owner thread stack;
*/
typedef struct thread_t
{
    int (*func)(void*);
    void* __stack__;
    int stackSize;
    int priority;
    struct thread_t* prev;
    struct thread_t* next;
}thread_t;


extern int threadcreate(thread_t* thread,void* stack,int stackSize,int (*func)(void*));
extern int threadstart(thread_t* thread);
extern int threadstop(thread_t* thread);

#ifdef __THREAD_PRIVATE__
extern void __cpusavecontext(void* dataspace);
extern void __cpurestorecontext(void* dataspace);
extern void __initThreading();
extern thread_t mainThread;
#endif

#ifdef __cplusplus
}
#endif
#endif //THREAD_H












